#include <windows.h>

#include "CoProcLib.h"
#include "SysInfoLib.h"

/*----------------------------------------------------------------------
|       main
+---------------------------------------------------------------------*/
int WINAPI WinMainCRTStartup(HINSTANCE hInstance,      // handle to current instance
                   HINSTANCE hPrevInstance,  // handle to previous instance
                   LPWSTR lpCmdLine,		 // pointer to command line
                   int nCmdShow)             // show state of window
{
	TCHAR wcTemp;
	DWORD len,dwSerial,dwSize,dwType;
	DWORD cpuSpeed, ramSpeed, busSpeed;
	DWORD ret;

	BSPVER version={0};
	SYSTEMTIME date={0};
	TCHAR szBuildDate[64];
	TCHAR szBeta[32];
	TCHAR szTemp[16];
	BYTE  mac[6]={0};

	OSVERSIONINFO	osInfo;
	PROCESSOR_INFO	cpuInfo;
	HWCFG			hwInfo;

	wprintf(_T("Toradex System Info Library\n\r\n\r"));


	// Get Bootloader Version and Build Date:
	GetBootloaderInfo(&version,&date);

	if(version.Min>>16) 
		swprintf(szBeta, L"BETA, Build %d", version.Min>>16);
	else
		szBeta[0]=0;

	len=GetDateFormat(LOCALE_SYSTEM_DEFAULT, 0, &date, L"dd MMM yyyy", szBuildDate, 64)-1;
	szBuildDate[len++]=' ';
	len+=GetTimeFormat(LOCALE_SYSTEM_DEFAULT, LOCALE_NOUSEROVERRIDE|TIME_FORCE24HOURFORMAT, &date, NULL, &szBuildDate[len], 64)-1;
	lstrcat(&szBuildDate[len], L" (GMT)");

	wprintf(L"Bootloader\r\nID:\t%s\r\nVer:\t%d.%d %s\r\nBuilt:\t%s", version.ID==1 ? L"COLIBRI" : L"UNKNOWN", version.Maj, version.Min&0xffff, szBeta, szBuildDate);
	

	// Get Image Version and Build Date:
	GetImageInfo(&version,&date);
	if(version.Min>>16) 
		swprintf(szBeta, L"BETA, Build %d", version.Min>>16);
	else
		szBeta[0]=0;

	len=GetDateFormat(LOCALE_SYSTEM_DEFAULT, 0, &date, L"dd MMM yyyy", szBuildDate, 64)-1;
	szBuildDate[len++]=' ';
	len+=GetTimeFormat(LOCALE_SYSTEM_DEFAULT, LOCALE_NOUSEROVERRIDE|TIME_FORCE24HOURFORMAT, &date, NULL, &szBuildDate[len], 64)-1;
	lstrcat(&szBuildDate[len], L" (GMT)");

	wprintf(L"\r\n\r\nCurrent Image\r\nID:\t%s\r\nVer:\t%d.%d %s\r\nBuilt:\t%s", version.ID==1 ? L"COLIBRI" : L"UNKNOWN", version.Maj, version.Min&0xffff, szBeta, szBuildDate);


	// Get OS Version:
	if(GetOSVersion(&osInfo))
		wprintf(L"\r\n\r\nCurrent OS:\t%s %d.%d", osInfo.dwPlatformId == VER_PLATFORM_WIN32_CE ? L"Windows CE" : L"UNKNOWN OS", osInfo.dwMajorVersion, osInfo.dwMinorVersion);


	// Get Processor Type:
	switch(getPROCID())
	{
		case TYPE_PXA270:	swprintf(szTemp, L"Marvell PXA270"); break;	
		case TYPE_PXA300:	swprintf(szTemp, L"Marvell PXA300"); break;
		case TYPE_PXA310:	swprintf(szTemp, L"Marvell PXA310"); break;
		case TYPE_PXA320:	swprintf(szTemp, L"Marvell PXA320"); break;
		default:	swprintf(szTemp, L"unknown CPU");
	}
	wprintf(L"\r\n\r\nProcessor Type:\t%s", szTemp);

	
	// Get Processor Info:
	GetProcessorInfo(&cpuInfo);
	wprintf(L"\r\n\r\nProcessor Info:\t%s, %s, %s, %d", cpuInfo.szVendor, cpuInfo.szProcessorName, cpuInfo.szProcessCore, cpuInfo.dwClockSpeed);


	// Get System Speeds:
	GetSystemSpeeds(&cpuSpeed, &ramSpeed, &busSpeed);
	cpuSpeed = cpuSpeed/(1000*1000);
	ramSpeed = ramSpeed/(1000*1000);
	busSpeed = busSpeed/(1000*1000);
	wprintf(L"\r\n\r\nSystem Speeds:\tCPU: %d MHz, RAM: %d MHz, BUS: %d MHz", cpuSpeed, ramSpeed, busSpeed);


	//Set CPU Core Voltage
	if(0) {
		DWORD voltage;
		voltage=SetCoreVoltage(1450);
		wprintf(L"\r\nSet core voltage to \t%d mV", voltage );
	}


	//Set System Frequency
	switch(getPROCID()) {
		case TYPE_PXA270:
			ret=0;
			wprintf(L"\r\nDo you want to set CPU Speed to 520MHz (f) or 312MHz (s) ? ");
			wcTemp = getwchar(); fflush(stdin);
			switch(wcTemp) {
				case 'f':
					ret =SetFrequencyPXA270(16, 5, TRUE, FALSE, TRUE, TRUE, TRUE);
				break;
				case 's':
					ret =SetFrequencyPXA270(16, 3, TRUE, FALSE, TRUE, TRUE, TRUE);
				break;
				default:
					wprintf(L"\r\nInput not valid\n\r");
				break;
			}
		break;
	
		case TYPE_PXA320:
			wprintf(L"\r\nDo you want to set CPU Speed to 806MHz (f) or 403MHz (s) ? ");
			wcTemp = getwchar(); fflush(stdin);
			switch(wcTemp){
				case 'f':
					ret = SetFrequencyPXA3xx(31,2,208,312,208);
				break;
				case 's':
					ret = SetFrequencyPXA3xx(31,1,208,312,208);
				break;
				default:
					wprintf(L"\r\nInput not valid\n\r");
				break;
			}
		break;

		default: break;
	}	
	if(ret==0) wprintf(L"\r\nSet New FrequencyPXA3xx Error\t%d", ret );

	// Get System Speeds again:
	GetSystemSpeeds(&cpuSpeed, &ramSpeed, &busSpeed);
	cpuSpeed = cpuSpeed/(1000*1000);
	ramSpeed = ramSpeed/(1000*1000);
	busSpeed = busSpeed/(1000*1000);
	wprintf(L"\r\n\r\nSystem Speeds:\tCPU: %d MHz, RAM: %d MHz, BUS: %d MHz", cpuSpeed, ramSpeed, busSpeed);


	// Get Memory Size:
	wprintf(L"\r\n\r\nMemory Size:\t%d kB RAM", GetMemorySize()/1024 );
	

	// Get Flash Size:
	dwSize = GetFlashSize(&dwType);
	wprintf(L"\r\nFlash Size:\t%d MB %s", dwSize/(1024*1024), dwType == TYPE_NOR ? L"NOR Flash" : L"NAND Flash");


	// Get Board Serial:
	GetBoardSerial(&dwSerial);
	wprintf(L"\r\n\r\nBoard Serial:\t%d", dwSerial);


	// Get Board Version:
	if(GetBoardVersion(&hwInfo))
		wprintf(L"\r\nBoard Version:\t%d.%d %c", hwInfo.VerMaj, hwInfo.VerMin, hwInfo.Variant+0x41);
	else
		wprintf(L"\r\nBoard Version:\tnot supported");


	// Get MAC Address:
	GetMACAddress(mac);
	wprintf(L"\r\n\r\nBoard MAC:\t%x:%x:%x:%x:%x:%x", mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	

	// Set MAC Address:
	wprintf(L"\r\nDo you want to save a new MAC Address (y/n)? ");
	wcTemp = getwchar(); fflush(stdin);
	if(wcTemp == 'y')
	{
		memset(&mac,0,sizeof(mac)); // clear to be sure we write to empty space
		wprintf(L"Enter new MAC Address: ");
		len = wscanf(L"%x:%x:%x:%x:%x:%x",&mac[0],&mac[1],&mac[2],&mac[3],&mac[4],&mac[5]); fflush(stdin);
		
		if (len == 6)
		{
			SetMACAddress(mac); // save new mac...

			memset(&mac,0,sizeof(mac)); // clear to be sure we get the newone for Flash
			GetMACAddress(mac);
			wprintf(L"New Board MAC Address:  %x:%x:%x:%x:%x:%x", mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
		}
		else
		{
			wprintf(L"Wrong input format. %d Bytes detected. Try this format X:X:X:X:X:X, where X is a one Byte HEX-Value.", len);
		}
		wprintf(L"\r\n");
	}

	// end programm
	wprintf(L"\r\nPress any key to end program");
	wcTemp = getwchar();
	fflush(stdin);

    return(TRUE);
}